# -*- coding: utf-8 -*-

"""Top-level package for iex-api."""

__author__ = "Volpyx"
__email__ = "thomas.kluiters@gmail.com"
# Do not edit this string manually, always use bumpversion
# Details in CONTRIBUTING.md
__version__ = "0.0.9"


def get_module_version():
    return __version__



import asyncio
import datetime
from dataclasses import dataclass
from typing import Iterable
from itertools import chain
from dataclasses_json import dataclass_json, LetterCase

from iex_api.model.common import IEXTimeSeriesObject


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class EndOfDayOptions(IEXTimeSeriesObject):
    ask: float
    bid: float
    cfi_code: str
    close: float
    closing_price: float
    contract_description: str
    contract_name: str
    contract_size: int
    currency: str
    exchange_code: str
    exchange_MIC: str
    exercise_style: str
    expiration_date: datetime.date
    figi: str
    high: float
    is_adjusted: bool
    last_trade_date: datetime.date
    last_trade_time: datetime.time
    last_updated: datetime.date
    low: float
    margin_price: float
    open: float
    open_interest: int
    settlement_price: float
    side: str
    strike_price: float
    symbol: str
    type: str
    volume: int

    @classmethod
    async def latest(cls, symbol: str, *args, **kwargs) -> Iterable["EndOfDayOptions"]:
        if "n" in kwargs and kwargs["n"] != 1:
            raise ValueError("Latest end of day options only supports n=1")
        expiry_dates = await cls.api().perform_request(f"/stock/{symbol}/options/", str)
        return await asyncio.gather(
            *chain(
                asyncio.ensure_future(
                    cls.api().perform_request(
                        f"/stock/{symbol}/options/{expiry_date}", EndOfDayOptions
                    )
                )
                for expiry_date in expiry_dates
            )
        )



import datetime
from dataclasses import dataclass
from typing import List, Optional

from dataclasses_json import dataclass_json, LetterCase

from iex_api.model.common import SymbolMixin, IEXTimeSeriesObject


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class Financials(IEXTimeSeriesObject):
    EBITDA: int
    accounts_payable: int
    capital_surplus: int
    cash_change: int
    cash_flow: int
    cash_flow_financing: int
    changes_in_inventories: int
    changes_in_receivables: int
    common_stock: int
    cost_of_revenue: int
    currency: str
    current_assets: int
    current_cash: int
    current_debt: int
    current_long_term_debt: int
    depreciation: int
    dividends_paid: int
    ebit: int
    exchange_rate_effect: int
    fiscal_date: datetime.date
    goodwill: int
    gross_profit: int
    income_tax: int
    intangible_assets: int
    interest_income: int
    inventory: int
    investing_activity_other: int
    investments: int
    longTerm_debt: int
    longTerm_investments: int
    minority_interest: int
    net_borrowings: int
    net_income: int
    net_income_basic: int
    net_tangible_assets: int
    operating_expense: int
    operating_income: int
    operating_revenue: int
    other_assets: int
    other_current_assets: int
    other_current_liabilities: int
    other_income_expense_net: int
    other_liabilities: int
    pretax_income: int
    property_plant_equipment: int
    receivables: int
    report_date: datetime.date
    research_and_development: int
    retained_earnings: int
    revenue: int
    selling_general_and_admin: int
    shareholder_equity: int
    shortTerm_debt: int
    shortTerm_investments: int
    symbol: str
    total_assets: int
    total_cash: int
    total_debt: int
    total_investing_cash_flows: int
    total_liabilities: int
    total_revenue: int
    treasury_stock: int

    ID = "FINANCIALS"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class CashFlow(IEXTimeSeriesObject):
    capital_expenditures: int
    cash_change: int
    cash_flow: int
    cash_flow_financing: int
    changes_in_inventories: int
    changes_in_receivables: int
    currency: str
    depreciation: int
    dividends_paid: int
    exchange_rate_effect: int
    fiscal_date: datetime.date
    investing_activity_other: int
    investments: int
    net_borrowings: int
    net_income: int
    other_financing_cash_flows: int
    report_date: datetime.date
    symbol: str
    total_investing_cash_flows: int

    ID = "CASH_FLOW"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class Income(IEXTimeSeriesObject):
    report_date: datetime.date
    fiscal_date: datetime.date
    currency: str
    total_revenue: int
    cost_of_revenue: int
    gross_profit: int
    research_and_development: int
    selling_general_and_admin: int
    operating_expense: int
    operating_income: int
    other_income_expense_net: int
    ebit: int
    interest_income: int
    pretax_income: int
    income_tax: int
    minority_interest: int
    net_income: int
    net_incomeBasic: int

    ID = "INCOME"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class Company(SymbolMixin):
    symbol: str
    company_name: str
    exchange: str
    industry: str
    website: str
    description: str
    CEO: str
    security_name: str
    issue_type: str
    sector: str
    primary_sic_code: int
    employees: int
    tags: List[str]
    address: str
    address2: Optional[str]
    state: str
    city: str
    zip: str
    country: str
    phone: str

    PATH = "company"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class KeyStats(SymbolMixin):
    company_name: str
    marketcap: int
    week52high: float
    week52low: float
    week52change: float
    shares_outstanding: float
    avg10_volume: int
    avg30_volume: int
    day200_moving_avg: int
    day50_moving_avg: int
    employees: int
    ttmEPS: float
    ttm_dividend_rate: float
    dividend_yield: float
    next_dividend_date: datetime.date
    ex_dividend_date: datetime.date
    next_earnings_date: datetime.date
    pe_ratio: float
    beta: float
    max_change_percent: float
    year5_change_percent: float
    year2_change_percent: float
    year1_change_percent: float
    ytd_change_percent: float
    month6_change_percent: float
    month3_change_percent: float
    month1_change_percent: float
    day30Change_percent: float
    day5Change_percent: float

    PATH = "stats"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class AdvancedStats(KeyStats):
    total_cash: int
    current_debt: int
    revenue: int
    gross_profit: int
    total_revenue: int
    EBITDA: int
    revenue_per_share: float
    revenue_per_employee: float
    debt_to_equity: float
    profit_margin: float
    enterprise_value: int
    enterprise_value_to_revenue: float
    price_to_sales: float
    price_to_book: float
    forward_PERatio: float
    peg_patio: float
    pe_high: float
    pe_low: float
    week52high_date: datetime.date
    week52low_date: datetime.date
    put_call_ratio: float

    PATH = "advanced-stats"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class Logo(SymbolMixin):
    url: str

    PATH = "logo"



import datetime
from dataclasses import dataclass
from typing import Optional

from dataclasses_json import LetterCase, dataclass_json

from iex_api.api.model.model import IEXRange
from iex_api.model.common import IEXTimeSeriesObject, SymbolMixin


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class IntraDayStockPrice:
    date: datetime.date
    minute: datetime.time
    market_average: float
    market_number_of_trades: int
    market_open: float
    market_close: float
    market_high: float
    market_low: float
    market_volume: int
    market_change_over_time: float
    change_over_time: float
    label: str


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class HistoricalStockPrice(IEXTimeSeriesObject, SymbolMixin):
    symbol: str
    close: float
    high: float
    low: float
    open: float
    volume: int
    change_over_time: float
    market_change_over_time: float
    u_open: Optional[float]
    u_close: Optional[float]
    u_high: Optional[float]
    u_low: Optional[float]
    u_volume: Optional[int]
    f_open: Optional[float]
    f_close: Optional[float]
    f_high: Optional[float]
    f_flow: Optional[float]
    f_volume: Optional[int]
    label: str
    change: Optional[float]
    change_percent: Optional[float]

    @classmethod
    async def from_symbol(
        cls, symbol: str, range: IEXRange, **kwargs
    ) -> "HistoricalStockPrice":
        return await cls.api().chart(symbol, range, cls, **kwargs)



import datetime
from dataclasses import dataclass

__all__ = [
    "today",
    "tomorrow",
    "yesterday",
    "year_to_date",
    "last_week",
    "last_month",
    "last_quarter",
    "this_week",
    "this_month",
    "this_quarter",
    "next_week",
    "next_month",
    "next_quarter",
    "d",
    "w",
    "m",
    "q",
    "y",
    "IEXRange",
    "TimeSeriesRequest",
]

from iex_api.api.util import convert_date_to_string


@dataclass
class IEXRange:
    value: str
    requires_calendar: bool

    def minute_intervals(self):
        return IEXRange(
            value=f"{self.value}m", requires_calendar=self.requires_calendar
        )


max = IEXRange("max", False)

today = IEXRange("today", False)
tomorrow = IEXRange("tomorrow", True)
yesterday = IEXRange("yesterday", False)
year_to_date = IEXRange("ytd", False)
last_week = IEXRange("last-week", False)
last_month = IEXRange("last-month", False)
last_quarter = IEXRange("last-quarter", False)
this_week = IEXRange("this-week", True)
this_month = IEXRange("this-month", True)
this_quarter = IEXRange("this-quarter", True)
next_week = IEXRange("next-week", True)
next_month = IEXRange("next-month", True)
next_quarter = IEXRange("next-quarter", True)


def date(date: datetime.date) -> IEXRange:
    return IEXRange(convert_date_to_string(date), False)


def d(amount: int) -> IEXRange:
    return IEXRange(f"{amount}d", False)


def w(amount: int) -> IEXRange:
    return IEXRange(f"{amount}w", False)


def m(amount: int) -> IEXRange:
    return IEXRange(f"{amount}m", False)


def q(amount: int) -> IEXRange:
    return IEXRange(f"{amount}q", False)


def y(amount: int) -> IEXRange:
    return IEXRange(f"{amount}y", False)


@dataclass
class TimeSeriesRequest:
    range: IEXRange = None
    calendar: bool = None
    limit: int = 1
    date_field: datetime.date = None
    from_date: datetime.date = None
    to_date: datetime.date = None
    on_date: datetime.date = None
    last: int = None
    first: int = None



import asyncio
from dataclasses import dataclass
import datetime
from itertools import chain
from typing import Optional, List

from dataclasses_json import dataclass_json, LetterCase

from iex_api.api.api import IEXApi
from iex_api.api.model.model import TimeSeriesRequest


class IEXBaseMixin:
    _API = None

    @classmethod
    def api(cls):
        if IEXBaseMixin._API is None:
            IEXBaseMixin._API = IEXApi.from_env()
        return IEXBaseMixin._API


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class IEXTimeSeriesObject(IEXBaseMixin):
    id: str
    key: str
    source: Optional[str]
    sub_key: Optional[str]
    date: Optional[datetime.date]
    updated: Optional[datetime.date]

    @classmethod
    async def all(cls, key: str, sub_key: str = None, output_constructor=list):
        return await cls.series(
            key, sub_key, TimeSeriesRequest(last=1_000_000), output_constructor
        )

    @classmethod
    async def latest(
        cls, key: str, sub_key: str = None, n: int = 1, output_constructor=list
    ):
        data = await cls.series(
            key,
            sub_key,
            TimeSeriesRequest(last=n),
            output_constructor=output_constructor,
        )
        return data[0] if n == 1 else data

    @classmethod
    async def series(
        cls,
        key: str,
        sub_key: str,
        request: TimeSeriesRequest = None,
        output_constructor=list,
    ):
        response = await cls.api().perform_time_series_request(
            cls.ID, key, cls, sub_key, request
        )
        return output_constructor(response)


@dataclass(frozen=True)
class SymbolMixin(IEXBaseMixin):
    @classmethod
    async def from_symbol(cls, symbol: str, *args, **kwargs):
        return await cls.api().perform_request(f"/stock/{symbol}/{cls.PATH}", cls)

    @classmethod
    async def from_symbols(cls, symbols: List[str]):
        all_data = await asyncio.gather(
            *chain(
                asyncio.ensure_future(
                    cls.api().perform_request(
                        "/stock/market/batch",
                        dict,
                        {
                            "symbols": ",".join(symbols[start : start + 100]),
                            "types": cls.PATH,
                        },
                    )
                )
                for start in range(0, len(symbols), 100)
            )
        )

        def extract_parts(data: dict):
            return (
                cls.from_dict(part[cls.PATH.lower()], infer_missing=True)
                for part in list(data.values())
            )

        return (item for sublist in map(extract_parts, all_data) for item in sublist)


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class Symbol(IEXBaseMixin):
    symbol: str
    exchange: str
    name: str
    date: datetime.date
    type: str
    iex_id: str
    region: str
    currency: str
    is_enabled: bool

    @classmethod
    async def get_all_symbols_for_region(cls, region: str) -> List["Symbol"]:
        return await cls.api().perform_request(
            f"/ref-data/region/{region}/symbols", cls
        )



import os
import aiohttp
import datetime

from dataclasses import dataclass

from typing import TypeVar, Generic, List, Union

from aiohttp import ClientResponse
from tenacity import retry, stop_after_attempt

from iex_api.api.model.model import IEXRange, TimeSeriesRequest
from iex_api.api.util import convert_date_to_string


def _extract_messages_used(response: ClientResponse) -> int:
    return int(response.headers.get("iexcloud-messages-used", "0"))


T = TypeVar("T")


@dataclass
class IEXApiConfiguration:
    api_token: str
    api_url: str

    retry_count: int = 10
    backoff_factor: float = 0.5

    def __post_init__(self):
        if self.api_url.endswith("/"):
            self.api_url = self.api_url[: len(self.api_url) - 1]

    @classmethod
    def from_env(cls):
        return cls(os.environ["IEX_API_TOKEN"], os.environ["IEX_API_URL"])


class IEXApi:
    def __init__(self, configuration: IEXApiConfiguration):
        self.configuration = configuration
        self._messages_used = 0

        self.perform_request = retry(
            self.perform_request,
            mulitplier=configuration.backoff_factor,
            stop=stop_after_attempt(configuration.retry_count),
        )

    @property
    def message_count(self):
        return self._messages_used

    @classmethod
    def from_env(cls):
        return cls(IEXApiConfiguration.from_env())

    async def chart(
        self,
        symbol: str,
        range: IEXRange,
        cls: T,
        chart_close_only: bool = None,
        chart_by_day: bool = None,
        chart_simplify: bool = None,
        chart_interval: int = None,
        change_from_close: bool = None,
        chart_last: int = None,
        display_percent: bool = None,
        exact_date: datetime.date = None,
        sort: bool = None,
        include_today: bool = None,
    ) -> List[T]:
        params = {}
        if chart_close_only is not None:
            params["chartCloseOnly"] = chart_close_only
        if chart_by_day is not None:
            params["chartByDay"] = chart_by_day
        if chart_simplify is not None:
            params["chartSimplify"] = chart_simplify
        if chart_interval is not None:
            params["chartInterval"] = chart_interval
        if change_from_close is not None:
            params["changeFromClose"] = change_from_close
        if chart_last is not None:
            params["chartLast"] = chart_last
        if display_percent is not None:
            params["displayPercent"] = display_percent
        if exact_date is not None:
            params["exactDate"] = convert_date_to_string(exact_date)
        if sort is not None:
            params["sort"] = sort
        if include_today is not None:
            params["includeToday"] = include_today

        return await self.perform_request(
            f"/stock/{symbol}/chart/{range.value}", cls, params
        )

    async def perform_time_series_request(
        self,
        id: str,
        key: str,
        cls: Generic[T],
        sub_key: str = None,
        request: TimeSeriesRequest = None,
    ) -> List[T]:
        params = {}
        if request:
            assert (
                not request.range
                or request.range.requires_calendar
                and request.calendar
            ), f"Calendar must be set to True if using {range}"
            assert request.limit >= 1, "Limit must be > 1"

            if request.range is not None:
                params["range"] = request.range.value
            if request.first is not None:
                params["first"] = request.first
            if request.last is not None:
                params["last"] = request.last
            if request.date_field is not None:
                params["dateField"] = convert_date_to_string(request.date_field)
            if request.to_date is not None:
                params["to"] = convert_date_to_string(request.to_date)
            if request.from_date is not None:
                params["from"] = convert_date_to_string(request.from_date)
            if request.on_date is not None:
                params["on"] = convert_date_to_string(request.on_date)

        path = f"/time-series/{id}/{key}"
        if sub_key is not None:
            path = f"{path}/{sub_key}"
        return await self.perform_request(path, cls, params)

    async def perform_request(
        self, path: str, cls: Generic[T], parameters: dict = None
    ) -> T:
        if parameters is None:
            parameters = {}
        if not path.startswith("/"):
            path = "/" + path

        params = dict(parameters, token=self.configuration.api_token)
        url = self.configuration.api_url + path

        if url.endswith("/"):
            url = url[: len(url) - 1]

        async with aiohttp.ClientSession() as session:
            async with session.get(url, params=params) as response:
                self._messages_used = self._messages_used + _extract_messages_used(
                    response
                )

                if response.status == 429:
                    response.raise_for_status()
                data = await response.json()
                if cls == dict:
                    return data
                if isinstance(data, list):

                    def parse(source: Union[dict, str]):

                        if isinstance(source, str):
                            return source
                        return cls.from_dict(source, infer_missing=True)

                    return map(parse, data)
                return cls.from_dict(data, infer_missing=True)



import datetime
from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase

from iex_api.model.common import IEXTimeSeriesObject


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class BasicDividend(IEXTimeSeriesObject):
    amount: float
    currency: str
    declared_date: datetime.date
    description: str
    ex_date: datetime.date
    flag: str
    frequency: str
    payment_date: datetime.date
    record_date: datetime.date
    refid: int
    symbol: str

    ID = "DIVIDENDS"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(frozen=True)
class AdvancedDividend(IEXTimeSeriesObject):
    symbol: str
    ex_date: datetime.date
    record_date: datetime.date
    payment_date: datetime.date
    announce_date: datetime.date
    currency: str
    frequency: str
    amount: float
    description: str
    flag: str
    security_type: str
    notes: str
    figi: str
    last_updated: datetime.date
    country_code: str
    par_value: str
    par_valueCurrency: str
    net_amount: float
    gross_amount: float
    marker: str
    tax_rate: float
    from_factor: float
    to_factor: float
    adr_fee: float
    coupon: float
    declared_currency_CD: str
    declared_gross_amount: float
    is_net_investment_income: bool
    is_DAP: bool
    is_approximate: bool
    fx_date: datetime.date
    second_payment_date: datetime.date
    second_ex_date: datetime.date
    fiscal_year_end_date: datetime.date
    period_end_date: datetime.date
    optional_election_date: datetime.date
    to_date: datetime.date
    registration_date: datetime.date
    installment_pay_date: datetime.date
    declared_date: datetime.date
    refid: int
    created: datetime.date

    ID = "ADVANCED_DIVIDENDS"
